/*
 * re-generate umsdos_fs.h #defines (regen part #2)
 *
 */
 

 
#include "umsdos_fs.h" 

#define DDEF(x) printf("#define " #x " %d\n", x);
#define SDEF(x) printf("#define " #x " \"%s\"\n", x);
 
int main ()
{
	printf ("/* This is autogenerated file, do not edit! */\n\n#include \"umsdos_fs2.h\"\n/*\n * We should actually parse and regenerate umsdos_fs.h,\n * but this is hand made for now for 2.2.9 kernel\n */\n");
	
	
	DDEF (UMSDOS_VERSION)
	DDEF (UMSDOS_RELEASE)

	SDEF (UMSDOS_EMD_FILE)
	DDEF (UMSDOS_EMD_NAMELEN)
	DDEF (UMSDOS_MAXNAME)
	SDEF (UMSDOS_PSDROOT_NAME)
	DDEF (UMSDOS_PSDROOT_LEN)

	DDEF (UMSDOS_REC_SIZE)
	DDEF (UMSDOS_HIDDEN)
	DDEF (UMSDOS_HLINK)
	
	DDEF (UMSDOS_READDIR_DOS)
	DDEF (UMSDOS_UNLINK_DOS)
	DDEF (UMSDOS_RMDIR_DOS)
	DDEF (UMSDOS_STAT_DOS)
	DDEF (UMSDOS_CREAT_EMD)
	DDEF (UMSDOS_UNLINK_EMD)
	DDEF (UMSDOS_READDIR_EMD)
	DDEF (UMSDOS_GETVERSION)
	DDEF (UMSDOS_INIT_EMD)
	DDEF (UMSDOS_DOS_SETUP)
	DDEF (UMSDOS_RENAME_DOS)
	
	
	return 0;
}

/*
we need those structs and all subelements in addition to above #defines:
umsdos_dirent
umsdos_info
umsdos_ioctl
*/
